"use strict";

exports.__esModule = true;
exports.watchCol = watchCol;
exports.unwatchCol = unwatchCol;
exports.destroyCol = destroyCol;
exports.unwatchAllCol = unwatchAllCol;
exports.unwatchAllCols = unwatchAllCols;
exports.getLocation = exports.getRef = exports.unWatch = exports.destroy = exports.childRemoved = exports.childChanged = exports.childAdded = exports.initialize = void 0;

var types = _interopRequireWildcard(require("./types"));

var selectors = _interopRequireWildcard(require("./selectors"));

var initSelectors = _interopRequireWildcard(require("../initialization/selectors"));

var _actions = require("../errors/actions");

var _actions2 = require("../loadings/actions");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var initialize = function initialize(list, location, path, locationValue, append) {
  return {
    type: types.INIIALIZE,
    payload: list,
    path: path,
    location: location,
    append: append,
    locationValue: locationValue
  };
};

exports.initialize = initialize;

var childAdded = function childAdded(child, location) {
  return {
    type: types.CHILD_ADDED,
    payload: child,
    location: location
  };
};

exports.childAdded = childAdded;

var childChanged = function childChanged(child, location) {
  return {
    type: types.CHILD_CHANGED,
    payload: child,
    location: location
  };
};

exports.childChanged = childChanged;

var childRemoved = function childRemoved(child, location) {
  return {
    type: types.CHILD_REMOVED,
    payload: child,
    location: location
  };
};

exports.childRemoved = childRemoved;

var destroy = function destroy(location) {
  return {
    type: types.DESTROY,
    location: location
  };
};

exports.destroy = destroy;

var unWatch = function unWatch(path) {
  return {
    type: types.UNWATCH,
    path: path
  };
};

exports.unWatch = unWatch;

var getPath = function getPath(ref) {
  return ref.path;
};

var getRef = function getRef(firebaseApp, path) {
  if (typeof path === 'string' || path instanceof String) {
    return firebaseApp.firestore().collection(path);
  } else {
    return path;
  }
};

exports.getRef = getRef;

var getLocation = function getLocation(path) {
  if (typeof path === 'string' || path instanceof String) {
    return path;
  } else {
    return getPath(path);
  }
};

exports.getLocation = getLocation;

function watchCol(firebaseApp, firebasePath, reduxPath, append) {
  if (reduxPath === void 0) {
    reduxPath = false;
  }

  if (append === void 0) {
    append = false;
  }

  var ref = getRef(firebaseApp, firebasePath);
  var path = getLocation(firebasePath);
  var location = reduxPath || path;
  return function (dispatch, getState) {
    var isInitialized = initSelectors.isInitialised(getState(), path, location);
    var initialized = false;

    if (!isInitialized) {
      dispatch((0, _actions2.logLoading)(location));
      var unsub = ref.onSnapshot(function (snapshot) {
        if (snapshot.size === 0) {
          dispatch((0, _actions2.clearLoading)(location));
        }

        snapshot.docChanges().forEach(function (change) {
          if (change.type === 'added') {
            if (initialized) {
              dispatch(childAdded({
                id: change.doc.id,
                data: change.doc.data()
              }, location));
            } else {
              initialized = true;
              dispatch(initialize([{
                id: change.doc.id,
                data: change.doc.data()
              }], location, path, unsub, append));
            }
          }

          if (change.type === 'modified') {
            dispatch(childChanged({
              id: change.doc.id,
              data: change.doc.data()
            }, location));
          }

          if (change.type === 'removed') {
            dispatch(childRemoved({
              id: change.doc.id,
              data: change.doc.data()
            }, location));
          }
        });
      }, function (err) {
        console.error(err);
        dispatch((0, _actions.logError)(location, err));
      });
    }
  };
}

function unwatchCol(firebaseApp, firebasePath) {
  return function (dispatch, getState) {
    var location = firebasePath;
    var allInitializations = selectors.getAllInitializations(getState());
    var unsubs = allInitializations[location];

    if (unsubs) {
      Object.keys(unsubs).map(function (key) {
        var unsub = unsubs[key];

        if (typeof unsub === 'function') {
          unsub();
        }

        dispatch(unWatch(location));
      });
    }
  };
}

function destroyCol(firebaseApp, firebasePath, reduxPath) {
  if (reduxPath === void 0) {
    reduxPath = false;
  }

  return function (dispatch, getState) {
    var location = reduxPath || getLocation(firebasePath);
    var locations = getState().initialization[location];
    dispatch(unWatch(location));
    dispatch(destroy(location));

    if (reduxPath) {
      dispatch(destroy(reduxPath));
      unwatchCol(firebaseApp, reduxPath);
    } else if (locations) {
      Object.keys(locations).forEach(function (location) {
        unwatchCol(firebaseApp, location);
        dispatch(destroy(location));
      });
    }
  };
}

function unwatchAllCol(firebaseApp, path) {
  return function (dispatch, getState) {
    var allLists = selectors.getAllCols(getState());
    Object.keys(allLists).forEach(function (key, index) {
      unwatchCol(firebaseApp, key);
      dispatch(unWatch(key));
    });
  };
}

function unwatchAllCols(firebaseApp, path) {
  return function (dispatch, getState) {
    var allColls = selectors.getAllCols(getState());
    Object.keys(allColls).forEach(function (key, index) {
      unwatchCol(firebaseApp, key);
      dispatch(destroyCol(firebaseApp, allColls[index]));
    });
  };
}