"use strict";

exports.__esModule = true;
exports.onConnectionStateChange = onConnectionStateChange;
exports.initConnection = initConnection;
exports.unsubscribeConnection = unsubscribeConnection;

var types = _interopRequireWildcard(require("./types"));

var _actions = require("../errors/actions");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function onConnectionStateChange(isConnected) {
  return {
    type: types.ON_CONNECTION_STATE_CHANGED,
    payload: {
      isConnected: isConnected
    }
  };
}

function initConnection(firebaseApp, onChange) {
  return function (dispatch) {
    firebaseApp.database().ref('.info/connected').on('value', function (snapshot) {
      dispatch(onConnectionStateChange(snapshot.val()));

      if (onChange !== undefined && onChange instanceof Function) {
        onChange(snapshot.val());
      }
    }, function (err) {
      console.error(err);
      dispatch((0, _actions.logError)('.info/connected', err));
    });
  };
}

function unsubscribeConnection(firebaseApp) {
  return function (dispatch) {
    firebaseApp.database().ref('.info/connected').off();
    dispatch(onConnectionStateChange(false));
  };
}