"use strict";

exports.__esModule = true;
exports.watchDoc = watchDoc;
exports.unwatchDoc = unwatchDoc;
exports.destroyDoc = destroyDoc;
exports.unwatchAllDocs = unwatchAllDocs;
exports.getLocation = exports.getRef = exports.unWatch = exports.destroy = exports.valueChanged = void 0;

var types = _interopRequireWildcard(require("./types"));

var selectors = _interopRequireWildcard(require("./selectors"));

var initSelectors = _interopRequireWildcard(require("../initialization/selectors"));

var _actions = require("../errors/actions");

var _actions2 = require("../loadings/actions");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var valueChanged = function valueChanged(value, location, path, locationValue) {
  return {
    type: types.VALUE_CHANGED,
    payload: value,
    path: path,
    location: location,
    locationValue: locationValue
  };
};

exports.valueChanged = valueChanged;

var destroy = function destroy(location) {
  return {
    type: types.DESTROY,
    location: location
  };
};

exports.destroy = destroy;

var unWatch = function unWatch(path) {
  return {
    type: types.UNWATCH,
    path: path
  };
};

exports.unWatch = unWatch;

var getRef = function getRef(firebaseApp, path) {
  if (typeof path === 'string' || path instanceof String) {
    return firebaseApp.firestore().doc(path);
  } else {
    return path;
  }
};

exports.getRef = getRef;

var getLocation = function getLocation(firebaseApp, path) {
  if (typeof path === 'string' || path instanceof String) {
    return path;
  } else {
    return firebaseApp.firestore().doc(path).path;
  }
};

exports.getLocation = getLocation;

function watchDoc(firebaseApp, firebasePath, reduxPath) {
  if (reduxPath === void 0) {
    reduxPath = false;
  }

  var ref = getRef(firebaseApp, firebasePath);
  var path = ref.path;
  var location = reduxPath || getLocation(firebaseApp, firebasePath);
  return function (dispatch, getState) {
    var isInitialized = initSelectors.isInitialised(getState(), location);

    if (!isInitialized) {
      dispatch((0, _actions2.logLoading)(location));
      var unsub = ref.onSnapshot(function (doc) {
        dispatch(valueChanged(doc.data(), location, path, unsub));
      }, function (err) {
        console.error(err);
        dispatch((0, _actions.logError)(location, err));
      });
    }
  };
}

function unwatchDoc(firebaseApp, path, reduxPath) {
  if (reduxPath === void 0) {
    reduxPath = false;
  }

  return function (dispatch, getState) {
    var location = reduxPath || path;
    var allInitializations = selectors.getAllInitializations(getState());
    var unsubs = allInitializations[location];

    if (unsubs) {
      Object.keys(unsubs).map(function (key) {
        var unsub = unsubs[key];

        if (typeof unsub === 'function') {
          unsub();
        }

        dispatch(unWatch(location));
      });
    }
  };
}

function destroyDoc(firebaseApp, path, reduxPath) {
  if (reduxPath === void 0) {
    reduxPath = false;
  }

  var location = reduxPath || path;
  return function (dispatch) {
    unwatchDoc(firebaseApp, location);
    dispatch(unWatch(location));
    dispatch(destroy(location));
  };
}

function unwatchAllDocs(firebaseApp) {
  return function (dispatch, getState) {
    var allPaths = selectors.getAllDocs(getState());
    Object.keys(allPaths).forEach(function (key, index) {
      unwatchDoc(firebaseApp, allPaths[index]);
      dispatch(unWatch(key));
    });
  };
}