"use strict";

exports.__esModule = true;
exports.watchList = watchList;
exports.unwatchList = unwatchList;
exports.destroyList = destroyList;
exports.unwatchAllLists = unwatchAllLists;
exports.destroyAllLists = destroyAllLists;
exports.getLocation = exports.getRef = exports.unWatch = exports.destroy = exports.childRemoved = exports.childChanged = exports.childAdded = exports.initialize = void 0;

var types = _interopRequireWildcard(require("./types"));

var selectors = _interopRequireWildcard(require("./selectors"));

var initSelectors = _interopRequireWildcard(require("../initialization/selectors"));

var _actions = require("../errors/actions");

var _actions2 = require("../loadings/actions");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var initialize = function initialize(list, location, path, append) {
  return {
    type: types.INIIALIZE,
    payload: list,
    path: path,
    location: location,
    append: append,
    locationValue: true
  };
};

exports.initialize = initialize;

var childAdded = function childAdded(child, location) {
  return {
    type: types.CHILD_ADDED,
    payload: child,
    location: location
  };
};

exports.childAdded = childAdded;

var childChanged = function childChanged(child, location) {
  return {
    type: types.CHILD_CHANGED,
    payload: child,
    location: location
  };
};

exports.childChanged = childChanged;

var childRemoved = function childRemoved(child, location) {
  return {
    type: types.CHILD_REMOVED,
    payload: child,
    location: location
  };
};

exports.childRemoved = childRemoved;

var destroy = function destroy(location) {
  return {
    type: types.DESTROY,
    location: location
  };
};

exports.destroy = destroy;

var unWatch = function unWatch(path) {
  return {
    type: types.UNWATCH,
    path: path
  };
};

exports.unWatch = unWatch;

var getPayload = function getPayload(snapshot) {
  return {
    key: snapshot.key,
    val: snapshot.val()
  };
};

var getRef = function getRef(firebaseApp, path) {
  if (typeof path === 'string' || path instanceof String) {
    return firebaseApp.database().ref(path);
  } else {
    return path;
  }
};

exports.getRef = getRef;

var getLocation = function getLocation(firebaseApp, path) {
  if (typeof path === 'string' || path instanceof String) {
    return path;
  } else {
    return path.toString().substring(firebaseApp.database().ref().root.toString().length);
  }
};

exports.getLocation = getLocation;

function watchList(firebaseApp, firebasePath, reduxPath, append) {
  if (reduxPath === void 0) {
    reduxPath = false;
  }

  if (append === void 0) {
    append = false;
  }

  var ref = getRef(firebaseApp, firebasePath);
  var path = ref.toString();
  var location = reduxPath || getLocation(firebaseApp, firebasePath);
  return function (dispatch, getState) {
    var initialized = false;
    var isInitialized = initSelectors.isInitialised(getState(), path, location);
    var persistetList = getState().lists ? getState().lists[location] : [];

    if (!isInitialized) {
      dispatch(initialize(persistetList, location, path, append));
      dispatch((0, _actions2.logLoading)(location));
      ref.once('value', function (snapshot) {
        initialized = true;
        var list = [];
        snapshot.forEach(function (childSnapshot) {
          var childKey = childSnapshot.key;
          var childData = childSnapshot.val();
          list.push({
            key: childKey,
            val: childData
          });
        });
        dispatch(initialize(list, location, path, append));
      }, function (err) {
        console.error(err);
        dispatch((0, _actions.logError)(location, err));
      });
      ref.on('child_added', function (snapshot) {
        if (initialized) {
          dispatch(childAdded(getPayload(snapshot), location));
        }
      }, function (err) {
        console.error(err);
        dispatch((0, _actions.logError)(location, err));
      });
      ref.on('child_changed', function (snapshot) {
        dispatch(childChanged(getPayload(snapshot), location));
      }, function (err) {
        console.error(err);
        dispatch((0, _actions.logError)(location, err));
      });
      ref.on('child_removed', function (snapshot) {
        dispatch(childRemoved(getPayload(snapshot), location));
      }, function (err) {
        console.error(err);
        dispatch((0, _actions.logError)(location, err));
      });
    }
  };
}

function unwatchList(firebaseApp, firebasePath) {
  return function (dispatch) {
    var ref = getRef(firebaseApp, firebasePath);
    ref.off();
    dispatch(unWatch(ref.toString()));
  };
}

function destroyList(firebaseApp, firebasePath, reduxPath) {
  if (reduxPath === void 0) {
    reduxPath = false;
  }

  return function (dispatch, getState) {
    var ref = getRef(firebaseApp, firebasePath);
    var locations = getState().initialization[ref.toString()];
    ref.off();
    dispatch(unWatch(ref.toString()));

    if (reduxPath) {
      dispatch(destroy(reduxPath));
    } else if (locations) {
      Object.keys(locations).forEach(function (location) {
        dispatch(destroy(location));
      });
    }
  };
}

function unwatchAllLists(firebaseApp, path) {
  return function (dispatch, getState) {
    var allLists = selectors.getAllLists(getState());
    Object.keys(allLists).forEach(function (key, index) {
      var ref = firebaseApp.database().ref(key);
      ref.off();
      dispatch(unWatch(ref.toString()));
    });
  };
}

function destroyAllLists(firebaseApp, path) {
  return function (dispatch, getState) {
    var allLists = selectors.getAllLists(getState());
    Object.keys(allLists).forEach(function (key, index) {
      var ref = firebaseApp.database().ref(key);
      ref.off();
      dispatch(destroyList(firebaseApp, ref.toString()));
    });
  };
}