"use strict";

exports.__esModule = true;
exports.onTokenChanged = onTokenChanged;
exports.onPermissionChanged = onPermissionChanged;
exports.onMessage = onMessage;
exports.clearMessage = clearMessage;
exports.onMessagingError = onMessagingError;
exports.initMessaging = initMessaging;

var types = _interopRequireWildcard(require("./types"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function onTokenChanged(token) {
  return {
    type: types.TOKEN_CHANGED,
    payload: {
      token: token,
      isInitialized: true
    }
  };
}

function onPermissionChanged(hasPermission, onMessage) {
  return {
    type: types.PERMISSION_CHANGED,
    payload: {
      hasPermission: hasPermission,
      isInitialized: true
    }
  };
}

function onMessage(message) {
  return {
    type: types.ON_MESSAGE,
    payload: {
      message: message
    }
  };
}

function clearMessage() {
  return {
    type: types.ON_CLEAR_MESSAGE
  };
}

function onMessagingError(error) {
  return {
    type: types.MESSAGING_ERROR,
    payload: {
      error: error
    }
  };
}

function initMessaging(firebaseApp, handleTokenChange, onMessageReceieved, onBackgroundMessageReceived) {
  return function (dispatch) {
    var messaging = firebaseApp.messaging();

    try {
      messaging.requestPermission().then(function () {
        return messaging.getToken();
      }).then(function (token) {
        if (handleTokenChange !== undefined && handleTokenChange instanceof Function) {
          handleTokenChange(token);
        }

        dispatch(onPermissionChanged(true));
        dispatch(onTokenChanged(token));
      })["catch"](function (error) {
        dispatch(onPermissionChanged(false));
        console.warn(error);
      });
    } catch (e) {
      dispatch(onPermissionChanged(false));
      console.warn(e);
    }

    messaging.onMessage(function (payload) {
      if (onMessageReceieved !== undefined && onMessageReceieved instanceof Function) {
        onMessageReceieved(payload);
      }

      dispatch(onMessage(payload));
    });
  };
}