"use strict";

exports.__esModule = true;
exports.watchPath = watchPath;
exports.unwatchPath = unwatchPath;
exports.destroyPath = destroyPath;
exports.unwatchAllPaths = unwatchAllPaths;
exports.unWatch = exports.destroy = exports.valueChanged = void 0;

var types = _interopRequireWildcard(require("./types"));

var selectors = _interopRequireWildcard(require("./selectors"));

var initSelectors = _interopRequireWildcard(require("../initialization/selectors"));

var _actions = require("../errors/actions");

var _actions2 = require("../loadings/actions");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var valueChanged = function valueChanged(value, location, path) {
  return {
    type: types.VALUE_CHANGED,
    payload: value,
    path: path,
    location: location,
    locationValue: true
  };
};

exports.valueChanged = valueChanged;

var destroy = function destroy(location) {
  return {
    type: types.DESTROY,
    location: location
  };
};

exports.destroy = destroy;

var unWatch = function unWatch(path) {
  return {
    type: types.UNWATCH,
    path: path
  };
};

exports.unWatch = unWatch;

function watchPath(firebaseApp, firebasePath, reduxPath, logLoad) {
  if (reduxPath === void 0) {
    reduxPath = false;
  }

  if (logLoad === void 0) {
    logLoad = true;
  }

  var location = reduxPath || firebasePath;
  return function (dispatch, getState) {
    var isInitialized = initSelectors.isInitialised(getState(), location);

    if (!isInitialized) {
      var ref = firebaseApp.database().ref(firebasePath);
      var path = ref.toString();

      if (logLoad) {
        dispatch((0, _actions2.logLoading)(location));
      }

      ref.on('value', function (snapshot) {
        dispatch(valueChanged(snapshot.val(), location, path));
      }, function (err) {
        dispatch((0, _actions.logError)(location, err));
      });
    }
  };
}

function unwatchPath(firebaseApp, path, reduxPath) {
  if (reduxPath === void 0) {
    reduxPath = false;
  }

  return function (dispatch) {
    var location = reduxPath || path;
    firebaseApp.database().ref(path).off();
    dispatch(unWatch(location));
  };
}

function destroyPath(firebaseApp, path, reduxPath) {
  if (reduxPath === void 0) {
    reduxPath = false;
  }

  var location = reduxPath || path;
  return function (dispatch) {
    firebaseApp.database().ref(path).off();
    dispatch(unWatch(location));
    dispatch(destroy(location));
  };
}

function unwatchAllPaths(firebaseApp) {
  return function (dispatch, getState) {
    var allPaths = selectors.getAllPaths(getState());
    Object.keys(allPaths).forEach(function (key, index) {
      firebaseApp.database().ref(allPaths[index]).off();
      dispatch(unWatch(key));
    });
  };
}